package com.orientdb.jsontest;

import java.util.UUID;

import org.junit.Before;
import org.junit.Test;

import com.orientechnologies.orient.core.db.ODatabasePool;
import com.orientechnologies.orient.core.db.ODatabaseSession;
import com.orientechnologies.orient.core.db.OrientDB;
import com.orientechnologies.orient.core.db.OrientDBConfig;
import com.orientechnologies.orient.core.sql.executor.OResultSet;

public class OdbJsonTest {
    private static final String DEBUG_MESSAGE = "\n[%s]\njson input  %s\njson output %s\n### %s";

    // TODO: change settings to some running odb instance
    private static final String DB_URL = "remote:127.0.0.1";
    private static final String DB_NAME = "";
    private static final String DB_USER = "";
    private static final String DB_PASSWORD = "";

    private ODatabasePool dbPool;

    @Before
    public void init() {
        OrientDB odb = new OrientDB(DB_URL, OrientDBConfig.defaultConfig());
        this.dbPool = new ODatabasePool(odb, DB_NAME, DB_USER, DB_PASSWORD);
    }

    @Test
    public void testJsonWithoutNull() {
        final String id = UUID.randomUUID().toString();
        final String jsonIn = String.format("{\"id\":\"%s\",\"list\":[{\"prop\":\"val\"},{\"prop\":\"val\"}]}", id);
        final String jsonOut = odbAndBack(id, jsonIn);
        System.out.println(String.format(DEBUG_MESSAGE,
                Thread.currentThread().getStackTrace()[1].getMethodName(),
                jsonIn, jsonOut, "OK"));
    }

    @Test
    public void testJsonWithNullInLastPosOnRoot() {
        final String id = UUID.randomUUID().toString();
        final String jsonIn = String
                .format("{\"id\":\"%s\",\"list\":[{\"prop\":\"val\"},{\"prop\":\"val\"}], \"prop\":null}", id);
        final String jsonOut = odbAndBack(id, jsonIn);
        System.out.println(String.format(DEBUG_MESSAGE,
                Thread.currentThread().getStackTrace()[1].getMethodName(),
                jsonIn, jsonOut, "OK"));
    }

    @Test
    public void testJsonWithNullNotInLastPosOnListItem() {
        final String id = UUID.randomUUID().toString();
        final String jsonIn = String.format(
                "{\"id\":\"%s\",\"list\":[{\"prop1\":\"val1\",\"prop2\":null,\"prop3\":\"val3\"},{\"prop\":\"val\"}]}",
                id);
        final String jsonOut = odbAndBack(id, jsonIn);
        System.out.println(String.format(DEBUG_MESSAGE,
                Thread.currentThread().getStackTrace()[1].getMethodName(),
                jsonIn, jsonOut, "OK"));
    }

    @Test
    public void testJsonWithNullInLastPosOnListItem() {
        // this one returns a broken json string. it's already written into the database
        // like it's returned later on in the query so it looks like the problem already
        // occurs down in the ORecordSerializerJSON/ORecordSerializerStringAbstract
        final String id = UUID.randomUUID().toString();
        final String jsonIn = String.format(
                "{\"id\":\"%s\",\"list\":[{\"prop1\":\"val1\",\"prop2\":null},{\"prop\":\"val\"}]}", id);
        final String jsonOut = odbAndBack(id, jsonIn);
        System.out.println(String.format(DEBUG_MESSAGE,
                Thread.currentThread().getStackTrace()[1].getMethodName(),
                jsonIn, jsonOut, "FAILED - this one writes a broken json string"));
    }

    private String odbAndBack(String id, String json) {
        try (ODatabaseSession session = dbPool.acquire()) {
            session.newElement("JsonTestGeneric").fromJSON(json).save();
            OResultSet rs = session.query(String.format("SELECT * FROM `JsonTestGeneric` WHERE `id` = '%s'", id));
            if (rs.hasNext()) {
                return rs.next().toJSON();
            }
        }
        // nevermind
        throw new RuntimeException();
    }
}
